<?php

use Drupal\Component\Utility\Html;
use Drupal\contact\ContactFormInterface as ContactFormEntityInterface;
use Drupal\contact\Entity\ContactForm as ContactFormEntity;
use Drupal\contact\Entity\Message as ContactMessageEntity;
use Drupal\contact\MessageForm as ContactMessageForm;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\RouteObjectInterface;
use Drupal\improvements\Ajax\FormResetCommand;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Route;

function improvements_contact_theme(): array {
  return [
    'contact_success_message' => [
      'variables' => [
        'message' => NULL,
      ],
    ],
  ];
}

/**
 * Implements hook_form_BASE_FORM_ID_alter(): contact_form_form.
 *
 * @see \Drupal\contact\ContactFormEditForm
 */
function improvements_form_contact_form_form_alter(array &$form, FormStateInterface $form_state): void {
  $contact_form_form_object = $form_state->getFormObject(); /** @var EntityFormInterface $contact_form_form_object */
  $contact_form_entity = $contact_form_form_object->getEntity(); /** @var ContactFormEntityInterface $contact_form_entity */

  $form['recipients_new'] = [
    '#type' => 'textfield',
    '#title' => $form['recipients']['#title'],
    '#description' => $form['recipients']['#description'],
    '#default_value' => $form['recipients']['#default_value'],
    '#required' => TRUE,
    '#weight' => 0.001,
  ];
  $form['recipients']['#default_value'] = 'dummy@dummy.dummy';
  $form['recipients']['#required'] = FALSE;
  $form['recipients']['#access'] = FALSE;

  $form['additional_settings'] = [
    '#type' => 'details',
    '#title' => t('Additional settings'),
  ];

  $form['additional_settings']['page_title'] = [
    '#type' => 'textfield',
    '#title' => t('Page title'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'page_title'),
    '#weight' => 1,
  ];

  $form['additional_settings']['text_before'] = [
    '#type' => 'textarea',
    '#title' => t('Text before form'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'text_before'),
    '#weight' => 2,
  ];

  $form['additional_settings']['text_after'] = [
    '#type' => 'textarea',
    '#title' => t('Text after form'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'text_after'),
    '#weight' => 3,
  ];

  $form['additional_settings']['name_label'] = [
    '#type' => 'textfield',
    '#title' => t('Name field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'name_label'),
    '#weight' => 4,
  ];

  $form['additional_settings']['mail_label'] = [
    '#type' => 'textfield',
    '#title' => t('E-mail field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'mail_label'),
    '#weight' => 5,
  ];

  $form['additional_settings']['subject_label'] = [
    '#type' => 'textfield',
    '#title' => t('Subject field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'subject_label'),
    '#weight' => 6,
  ];

  $form['additional_settings']['message_label'] = [
    '#type' => 'textfield',
    '#title' => t('Message field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'message_label'),
    '#weight' => 7,
  ];

  $form['additional_settings']['submit_button_text'] = [
    '#type' => 'textfield',
    '#title' => t('Submit button text'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'submit_button_text'),
    '#weight' => 8,
  ];

  $form['additional_settings']['name_placeholder'] = [
    '#type' => 'textfield',
    '#title' => t('Name field placeholder'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'name_placeholder'),
    '#weight' => 9,
  ];

  $form['additional_settings']['email_placeholder'] = [
    '#type' => 'textfield',
    '#title' => t('E-mail field placeholder'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'email_placeholder'),
    '#weight' => 10,
  ];

  $form['additional_settings']['email_subject'] = [
    '#type' => 'textfield',
    '#title' => t('E-mail subject'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'email_subject'),
    '#weight' => 11,
  ];

  $form['additional_settings']['required_fields'] = [
    '#type' => 'checkboxes',
    '#title' => t('Required fields'),
    '#options' => [
      'name' => t('Name'),
      'mail' => t('E-mail'),
      'subject' => t('Subject'),
      'message' => t('Message'),
    ],
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'required_fields', ['name', 'mail', 'subject', 'message']),
    '#weight' => 12,
  ];

  $form['additional_settings']['ajax_submit'] = [
    '#type' => 'checkbox',
    '#title' => t('AJAX submit'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit'),
    '#weight' => 13,
  ];

  $form['additional_settings']['ajax_submit_method'] = [
    '#type' => 'radios',
    '#title' => t('Show successful message in'),
    '#options' => [
      'form' => t('Form'),
      'dialog' => t('Dialog'),
    ],
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit_method', 'form'),
    '#states' => [
      'visible' => [
        ':input[name="ajax_submit"]' => [
          'checked' => TRUE,
        ],
      ],
    ],
    '#weight' => 14,
  ];

  $form['additional_settings']['dialog_title'] = [
    '#type' => 'textfield',
    '#title' => t('Dialog title'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'dialog_title'),
    '#states' => $form['additional_settings']['ajax_submit_method']['#states'],
    '#weight' => 15,
  ];

  $form['additional_settings']['hide_preview_button'] = [
    '#type' => 'checkbox',
    '#title' => t('Hide preview button'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'hide_preview_button'),
    '#weight' => 16,
  ];

  $form['#entity_builders'][] = 'improvements_contact_form_entity_builder';
  $form['actions']['submit']['#submit'][] = 'improvements_contact_form_form_submit';
}

/**
 * Contact form entity builder.
 *
 * @see improvements_form_contact_form_form_alter().
 */
function improvements_contact_form_entity_builder(string $entity_type, ContactFormEntityInterface $contact_form, array &$form, FormStateInterface $form_state): void {
  // Save recipients
  $recipients = array_map('trim', explode(',', $form_state->getValue('recipients_new')));
  $contact_form->setRecipients($recipients);

  // Save additional settings
  $simple_settings = [
    'page_title',
    'text_before',
    'text_after',
    'name_label',
    'mail_label',
    'subject_label',
    'message_label',
    'submit_button_text',
    'name_placeholder',
    'email_placeholder',
    'email_subject',
    'ajax_submit',
    'ajax_submit_method',
    'dialog_title',
    'hide_preview_button',
  ];
  foreach ($simple_settings as $setting_name) {
    $contact_form->setThirdPartySetting('improvements', $setting_name, $form_state->getValue($setting_name));
  }

  $contact_form->setThirdPartySetting('improvements', 'required_fields', Checkboxes::getCheckedCheckboxes($form_state->getValue('required_fields')));
}

/**
 * Custom submit callback for contact edit form.
 *
 * @see improvements_form_contact_form_edit_form_alter().
 */
function improvements_contact_form_form_submit(array $form, FormStateInterface $form_state): void {
  $contact_form_form_object = $form_state->getFormObject(); /** @var EntityFormInterface $contact_form_form_object */
  $contact_form_entity = $contact_form_form_object->getEntity(); /** @var ContactFormEntityInterface $contact_form_entity */
  $contact_form_entity_id = $contact_form_entity->id();
  $entity_field_manager = \Drupal::service('entity_field.manager');
  $contact_message_entity_field_definitions = $entity_field_manager->getFieldDefinitions('contact_message', $contact_form_entity_id);

  // Set new labels
  /** @see improvements_entity_bundle_field_info_alter() */
  /*foreach (['name', 'mail', 'subject', 'message'] as $field_name) {
    $field_config = $contact_message_field_definitions[$field_name]->getConfig($contact_form_entity_id);
    if (
      ($new_field_label = $form_state->getValue($field_name . '_label')) &&
      $new_field_label != $field_config->getLabel()
    ) {
      $field_config->setLabel(trim($new_field_label))->save();
    }
  }*/

  // Set required fields
  $required_fields = Checkboxes::getCheckedCheckboxes($form_state->getValue('required_fields'));
  foreach (['subject', 'message'] as $field_name) {
    $field_is_required = in_array($field_name, $required_fields);
    $contact_message_entity_field_definitions[$field_name]
      ->getConfig($contact_form_entity_id)
      ->setRequired($field_is_required)
      ->save();
  }
}

/**
 * Implements hook_form_BASE_FORM_ID_alter(): contact_message_form.
 *
 * @see \Drupal\contact\MessageForm::form()
 */
function improvements_form_contact_message_form_alter(array &$form, FormStateInterface $form_state, string $form_id): void {
  $contact_message_form_object = $form_state->getFormObject(); /** @var ContactMessageForm $contact_message_form_object */
  $contact_message_entity = $contact_message_form_object->getEntity(); /** @var ContactMessageEntity $contact_message_entity */
  $contact_form_entity = $contact_message_entity->getContactForm();
  $required_fields = $contact_form_entity->getThirdPartySetting('improvements', 'required_fields', []);

  // Add texts
  if ($text_before = $contact_form_entity->getThirdPartySetting('improvements', 'text_before')) {
    $form['text_before'] = [
      '#type' => 'item',
      '#markup' => $text_before,
      '#weight' => -100,
    ];
  }
  if ($text_after = $contact_form_entity->getThirdPartySetting('improvements', 'text_after')) {
    $form['text_after'] = [
      '#type' => 'item',
      '#markup' => $text_after,
      '#weight' => 100,
    ];
  }

  // Change labels
  if ($name_label = $contact_form_entity->getThirdPartySetting('improvements', 'name_label')) {
    $form['name']['#title'] = $name_label;
  }
  if ($mail_label = $contact_form_entity->getThirdPartySetting('improvements', 'mail_label')) {
    $form['mail']['#title'] = $mail_label;
  }

  // Set placeholder
  if ($name_placeholder = $contact_form_entity->getThirdPartySetting('improvements', 'name_placeholder')) {
    $form['name']['#placeholder'] = $name_placeholder;
  }
  if ($email_placeholder = $contact_form_entity->getThirdPartySetting('improvements', 'email_placeholder')) {
    $form['mail']['#placeholder'] = $email_placeholder;
  }

  // Restore "name" and "mail" elements for logged users
  if (\Drupal::currentUser()->isAuthenticated()) {
    if (isset($form['name'])) {
      $form['name']['#type'] = 'textfield';
      $form['name']['#default_value'] = $form['name']['#value'];
      unset($form['name']['#value']);
    }
    if (isset($form['mail'])) {
      $form['mail']['#type'] = 'email';
      $form['mail']['#default_value'] = $form['mail']['#value'];
      unset($form['mail']['#value']);
    }
  }

  // Change #required option
  if (isset($form['name'])) {
    $form['name']['#required'] = in_array('name', $required_fields);
  }
  if (isset($form['mail'])) {
    $form['mail']['#required'] = in_array('mail', $required_fields);
  }

  // Change submit button text
  if ($submit_button_text = $contact_form_entity->getThirdPartySetting('improvements', 'submit_button_text')) {
    $form['actions']['submit']['#value'] = $submit_button_text;
  }

  // Hide "preview" button
  if ($contact_form_entity->getThirdPartySetting('improvements', 'hide_preview_button')) {
    $form['actions']['preview']['#access'] = FALSE;
  }

  // AJAX submit
  if ($contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit')) {
    $form_html_id = Html::getId($contact_message_form_object->getFormId());
    $form['#id'] = $form_html_id;

    $form['actions']['submit']['#ajax'] = [
      'callback' => 'improvements_form_contact_message_form_ajax',
      'event' => 'click',
    ];
  }
}

/**
 * Contact form ajax submit callback.
 *
 * @see improvements_form_contact_message_form_alter()
 */
function improvements_form_contact_message_form_ajax(array &$form, FormStateInterface $form_state): Response {
  $response = new AjaxResponse();
  $form_selector = '#' . $form['#id'];

  if ($form_state->hasAnyErrors()) {
    // Update contact form
    $response->addCommand(new ReplaceCommand($form_selector, $form));
    // Add status messages in contact form
    $response->addCommand(new PrependCommand($form_selector, ['#type' => 'status_messages']));
  }
  else {
    // Delete status messages (warnings and errors are keep)
    \Drupal::messenger()->deleteByType(MessengerInterface::TYPE_STATUS);

    // Show success message
    $contact_form_object = $form_state->getFormObject(); /** @var ContactMessageForm $contact_form_object */
    $contact_message_entity = $contact_form_object->getEntity(); /** @var ContactMessageEntity $contact_message_entity */
    $contact_form_entity = $contact_message_entity->getContactForm();
    $complete_message = [
      'status_message' => [
        '#type' => 'status_messages'
      ],
      'success_message' => [
        '#theme' => 'contact_success_message',
        '#message' => Markup::create($contact_form_entity->getMessage()) ?: t('Form is submitted'),
      ],
    ];

    $ajax_submit_method = $contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit_method', 'form');

    // Show message instead form
    if ($ajax_submit_method == 'form') {
      $response->addCommand(new HtmlCommand($form_selector, $complete_message));
    }
    // Show message in dalog
    elseif ($ajax_submit_method == 'dialog') {
      $dialog_title = $contact_form_entity->getThirdPartySetting('improvements', 'dialog_title');
      $response->addCommand(new OpenModalDialogCommand($dialog_title, $complete_message));
      $response->addCommand(new FormResetCommand($form_selector));
    }

    \Drupal::moduleHandler()->alter([
      'conact_message_form_ajax_response',
      'conact_message_form_' . $contact_form_entity->id() . '_ajax_response',
    ], $response, $form, $form_state);
  }

  return $response;
}

/**
 * Implements hook_page_title_alter().
 */
function improvements_page_title_alter(&$title, Request $request, Route $route): void {
  $route_name = $request->attributes->get(RouteObjectInterface::ROUTE_NAME);

  if ($route_name == 'entity.contact_form.canonical') {
    $contact_form_entity = $request->attributes->get('contact_form'); /** @var ContactFormEntityInterface $contact_form_entity */
    if ($contact_form_page_title = $contact_form_entity->getThirdPartySetting('improvements', 'page_title')) {
      $title = $contact_form_page_title;
    }
  }
}

/**
 * Implements hook_mail_ID_alter(): contact_page_mail.
 */
function improvements_mail_contact_page_mail_alter(array &$message): void {
  $contact_form_entity = $message['params']['contact_form']; /** @var ContactFormEntityInterface $contact_form_entity */

  // Change contact message email subject
  if ($contact_form_email_subject = $contact_form_entity->getThirdPartySetting('improvements', 'email_subject')) {
    $message['subject'] = $contact_form_email_subject;
  }
}

/**
 * Implements hook_entity_bundle_field_info_alter().
 */
function improvements_entity_bundle_field_info_alter(array &$fields, EntityTypeInterface $entity_type, ?string $bundle) {
  // Change contact message entity fields labels
  // Use this method because method in improvements_contact_form_form_submit() not support translation
  if ($bundle && $fields && $entity_type->id() == 'contact_message') {
    $contact_form_entity = ContactFormEntity::load($bundle);
    $contact_form_entity_third_party_settings = $contact_form_entity->getThirdPartySettings('improvements');

    foreach (['name', 'mail', 'subject', 'message'] as $field_name) {
      if (
        !empty($contact_form_entity_third_party_settings[$field_name . '_label']) &&
        isset($fields[$field_name])
      ) {
        $fields[$field_name]->setLabel($contact_form_entity_third_party_settings[$field_name . '_label']);
      }
    }
  }
}
