<?php

use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Drupal\pathauto\Form\PatternEditForm;
use Drupal\pathauto\PathautoPatternInterface;

/**
 * Implements hook_form_FORM_ID_alter(): pathauto_pattern_form.
 */
function improvements_form_pathauto_pattern_form_alter(array &$form, FormStateInterface $form_state): void {
  $pathauto_patter_form_object = $form_state->getFormObject(); /** @var PatternEditForm $pathauto_patter_form_object */
  $pathauto_pattern_entity = $pathauto_patter_form_object->getEntity(); /** @var PathautoPatternInterface $pathauto_pattern_entity */

  if ($pathauto_pattern_entity->getType() == 'canonical_entities:taxonomy_term') {
    // Find term_depth condition
    $term_depth_condition = NULL;
    foreach ($pathauto_pattern_entity->getSelectionConditions() as $condition) {
      if ($condition->getPluginId() == 'term_depth') {
        $term_depth_condition = $condition;
        break;
      }
    }

    // Create term_depth condition if not exists
    if (!$term_depth_condition) {
      $condition_manager = \Drupal::service('plugin.manager.condition'); /** @var ConditionManager $condition_manager */
      $term_depth_condition = $condition_manager->createInstance('term_depth');
    }

    $form['pattern_container']['depth_condition'] = [
      '#type' => 'details',
      '#title' => t('Term depth condition'),
      '#tree' => TRUE,
    ];
    $form['pattern_container']['depth_condition'] = $term_depth_condition->buildConfigurationForm($form['pattern_container']['depth_condition'], $form_state);

    $form['#entity_builders'][] = 'improvements_form_pathauto_pattern_form_entity_builders';
  }
}

/**
 * Pathauto "pattern" form #entity_builders callback.
 *
 * @see improvements_form_pathauto_pattern_form_alter().
 */
function improvements_form_pathauto_pattern_form_entity_builders(string $entity_type, PathautoPatternInterface $pathauto_pattern, array &$form, FormStateInterface $form_state): void {
  $selected_depth = Checkboxes::getCheckedCheckboxes($form_state->getValue(['depth_condition', 'depth']));

  // Remove old condition
  foreach ($pathauto_pattern->getSelectionConditions() as $condition_id => $condition) {
    if ($condition->getPluginId() == 'term_depth') {
      $pathauto_pattern->removeSelectionCondition($condition_id);
    }
  }

  // Add new condition
  if ($selected_depth) {
    $pathauto_pattern->addSelectionCondition([
      'id' => 'term_depth',
      'depth' => $selected_depth,
      'negate' => FALSE,
    ]);
  }
}
