<?php

namespace Drupal\improvements;

use Drupal\contact\MessageInterface;
use Drupal\Core\Entity\ContentEntityStorageInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;

class ImprovementsHelper {

  /**
   * Return contact form.
   */
  public static function getContactForm(string $contact_form_id): array {
    $contact_message_storage = \Drupal::entityTypeManager()->getStorage('contact_message'); /** @var ContentEntityStorageInterface $contact_message_storage */
    $contact_message = $contact_message_storage->create(['contact_form' => $contact_form_id]); /** @var MessageInterface $contact_message */

    $entity_form_builder = \Drupal::service('entity.form_builder'); /** @var EntityFormBuilderInterface $entity_form_builder */
    $form = $entity_form_builder->getForm($contact_message);
    $form['#cache']['contexts'][] = 'user.permissions';

    return $form;
  }

  /**
   * Return textarea rows count by text length.
   */
  public static function getTextareaRowsCount(string $text, int $min_rows): int {
    $rows = substr_count($text, "\n");
    $rows = min($rows + 1, 28);
    return max($rows, $min_rows);
  }

  /**
   * Return e-mail for notifications.
   */
  public static function getEmailForNotifications(): string {
    $site_info = improvements_get_site_information();
    return $site_info['mail_for_notifications'] ?: $site_info['mail'];
  }

  /**
   * Return biild time.
   */
  public static function getBuildTime(): string {
    static $build_time;

    if ($build_time === NULL) {
      $build_time = date(DATE_ATOM, \Drupal::time()->getCurrentTime());
    }

    return $build_time;
  }

}
