<?php

namespace Drupal\improvements;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use Twig\TwigFunction;

/**
 * Twig extensions.
 */
class ImprovementsTwigExtension extends AbstractExtension {

  /**
   * {@inheritdoc}
   */
  public function getFunctions(): array {
    return [
      new TwigFunction('drupal_contact', '\Drupal\improvements\ImprovementsHelper::getContactForm'),
      new TwigFunction('entity_url', [$this, 'entityUrl']), // @TODO Remove after issue close https://www.drupal.org/project/drupal/issues/2907810
      new TwigFunction('preg_matches', [$this, 'pregMatches']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFilters(): array {
    return [
      new TwigFilter('format_size', 'format_size'),
      new TwigFilter('without_theme', [$this, 'withoutTheme']),
    ];
  }

  /**
   * Return entity url.
   */
  public function entityUrl(EntityInterface $entity, string $rel = 'canonical', array $options = []): Url {
    return $entity->toUrl($rel, $options);
  }

  /**
   * Call preg_match() and return matches.
   *
   * @return string|array|null
   */
  public function pregMatches(string $pattern, string $subject, int $match_index = NULL) {
    if (preg_match($pattern, $subject, $matches)) {
      if ($match_index !== NULL) {
        return $matches[$match_index] ?? NULL;
      }
      return $matches;
    }
    return NULL;
  }

  /**
   * Return render-array without #theme property.
   *
   * @return mixed
   */
  public function withoutTheme($element) {
    if (is_array($element) && isset($element['#theme'])) {
      unset($element['#theme']);
    }
    return $element;
  }

}
