<?php

namespace Drupal\improvements\Plugin\views\display_extender;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\display_extender\DisplayExtenderPluginBase;

/**
 * @ViewsDisplayExtender(
 *   id = "improvements_display_extender",
 *   title = @Translation("Improvements display extender"),
 *   no_ui = FALSE
 * )
 */
class ImprovementsDisplayExtender extends DisplayExtenderPluginBase {

  /**
   * {@inheritDoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    if ($form_state->get('section') == 'css_class') {
      $form['content_css_class'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Content CSS class name(s)'),
        '#description' => $this->t('Separate multiple classes by spaces.') . ' ' . t('Supports tokens.'),
        '#default_value' => $this->options['content_css_class'],
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state): void {
    if ($form_state->get('section') == 'css_class') {
      $this->options['content_css_class'] = $form_state->getValue('content_css_class');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function optionsSummary(&$categories, &$options): void {
    if (!empty($this->options['content_css_class'])) {
      $options['css_class']['value'] .= ' (' . $this->options['content_css_class'] . ')';
    }
  }

}
